// Code generated by sqlc. DO NOT EDIT.
// versions:
//   sqlc v1.29.0
// source: notes.sql

package data

import (
	"context"
	"time"

	"github.com/google/uuid"
)

const createNote = `-- name: CreateNote :one
INSERT INTO notes (user_id)
VALUES ($1)
RETURNING id, user_id, current_version, latest_version, expires_at, created_at, updated_at
`

func (q *Queries) CreateNote(ctx context.Context, userID uuid.UUID) (Note, error) {
	row := q.db.QueryRow(ctx, createNote, userID)
	var i Note
	err := row.Scan(
		&i.ID,
		&i.UserID,
		&i.CurrentVersion,
		&i.LatestVersion,
		&i.ExpiresAt,
		&i.CreatedAt,
		&i.UpdatedAt,
	)
	return i, err
}

const deleteExpiredNotes = `-- name: DeleteExpiredNotes :exec
DELETE FROM notes
WHERE expires_at < NOW()
`

func (q *Queries) DeleteExpiredNotes(ctx context.Context) error {
	_, err := q.db.Exec(ctx, deleteExpiredNotes)
	return err
}

const deleteNote = `-- name: DeleteNote :exec
DELETE FROM notes
WHERE id = $1 AND user_id = $2
`

type DeleteNoteParams struct {
	ID     uuid.UUID `json:"id"`
	UserID uuid.UUID `json:"user_id"`
}

func (q *Queries) DeleteNote(ctx context.Context, arg DeleteNoteParams) error {
	_, err := q.db.Exec(ctx, deleteNote, arg.ID, arg.UserID)
	return err
}

const getFullNote = `-- name: GetFullNote :one
SELECT 
    n.id AS note_id,
    n.user_id AS owner_id,
    nv.title, 
    nv.content, 
    nv.version_number, 
    nv.created_at AS version_created_at,
    n.expires_at AS note_expires_at,
    n.created_at AS note_created_at,
    n.updated_at AS note_updated_at
FROM notes n
JOIN note_versions nv 
    ON n.id = nv.note_id AND n.current_version = nv.version_number
WHERE n.id = $1
`

type GetFullNoteRow struct {
	NoteID           uuid.UUID  `json:"note_id"`
	OwnerID          uuid.UUID  `json:"owner_id"`
	Title            string     `json:"title"`
	Content          string     `json:"content"`
	VersionNumber    int32      `json:"version_number"`
	VersionCreatedAt *time.Time `json:"version_created_at"`
	NoteExpiresAt    *time.Time `json:"note_expires_at"`
	NoteCreatedAt    *time.Time `json:"note_created_at"`
	NoteUpdatedAt    *time.Time `json:"note_updated_at"`
}

func (q *Queries) GetFullNote(ctx context.Context, id uuid.UUID) (GetFullNoteRow, error) {
	row := q.db.QueryRow(ctx, getFullNote, id)
	var i GetFullNoteRow
	err := row.Scan(
		&i.NoteID,
		&i.OwnerID,
		&i.Title,
		&i.Content,
		&i.VersionNumber,
		&i.VersionCreatedAt,
		&i.NoteExpiresAt,
		&i.NoteCreatedAt,
		&i.NoteUpdatedAt,
	)
	return i, err
}

const listExpiredNotes = `-- name: ListExpiredNotes :many
SELECT 
    n.id AS note_id,
    n.user_id AS owner_id,
    nv.title,
    n.expires_at
FROM notes n
JOIN note_versions nv 
    ON n.id = nv.note_id AND n.current_version = nv.version_number
WHERE n.expires_at <= NOW()
ORDER BY n.expires_at
`

type ListExpiredNotesRow struct {
	NoteID    uuid.UUID  `json:"note_id"`
	OwnerID   uuid.UUID  `json:"owner_id"`
	Title     string     `json:"title"`
	ExpiresAt *time.Time `json:"expires_at"`
}

func (q *Queries) ListExpiredNotes(ctx context.Context) ([]ListExpiredNotesRow, error) {
	rows, err := q.db.Query(ctx, listExpiredNotes)
	if err != nil {
		return nil, err
	}
	defer rows.Close()
	var items []ListExpiredNotesRow
	for rows.Next() {
		var i ListExpiredNotesRow
		if err := rows.Scan(
			&i.NoteID,
			&i.OwnerID,
			&i.Title,
			&i.ExpiresAt,
		); err != nil {
			return nil, err
		}
		items = append(items, i)
	}
	if err := rows.Err(); err != nil {
		return nil, err
	}
	return items, nil
}

const listNotes = `-- name: ListNotes :many
SELECT 
    n.id AS note_id,
    n.user_id AS owner_id,
    nv.title,
    n.expires_at,
    n.updated_at
FROM notes n
JOIN note_versions nv 
    ON n.id = nv.note_id AND n.current_version = nv.version_number
WHERE n.user_id = $1
ORDER BY n.updated_at DESC
LIMIT $2 OFFSET $3
`

type ListNotesParams struct {
	UserID uuid.UUID `json:"user_id"`
	Limit  int32     `json:"limit"`
	Offset int32     `json:"offset"`
}

type ListNotesRow struct {
	NoteID    uuid.UUID  `json:"note_id"`
	OwnerID   uuid.UUID  `json:"owner_id"`
	Title     string     `json:"title"`
	ExpiresAt *time.Time `json:"expires_at"`
	UpdatedAt *time.Time `json:"updated_at"`
}

func (q *Queries) ListNotes(ctx context.Context, arg ListNotesParams) ([]ListNotesRow, error) {
	rows, err := q.db.Query(ctx, listNotes, arg.UserID, arg.Limit, arg.Offset)
	if err != nil {
		return nil, err
	}
	defer rows.Close()
	var items []ListNotesRow
	for rows.Next() {
		var i ListNotesRow
		if err := rows.Scan(
			&i.NoteID,
			&i.OwnerID,
			&i.Title,
			&i.ExpiresAt,
			&i.UpdatedAt,
		); err != nil {
			return nil, err
		}
		items = append(items, i)
	}
	if err := rows.Err(); err != nil {
		return nil, err
	}
	return items, nil
}

const setNoteExpiration = `-- name: SetNoteExpiration :exec
UPDATE notes 
SET expires_at = $1, updated_at = NOW() 
WHERE id = $2 AND user_id = $3
`

type SetNoteExpirationParams struct {
	ExpiresAt *time.Time `json:"expires_at"`
	ID        uuid.UUID  `json:"id"`
	UserID    uuid.UUID  `json:"user_id"`
}

func (q *Queries) SetNoteExpiration(ctx context.Context, arg SetNoteExpirationParams) error {
	_, err := q.db.Exec(ctx, setNoteExpiration, arg.ExpiresAt, arg.ID, arg.UserID)
	return err
}
